package com.lpsfr.lib;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.facebook.react.bridge.ActivityEventListener;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.bridge.WritableNativeMap;
import com.lpsfr.lib.utils.FileHelper;
import com.lpsfr.lib.utils.Logcat;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public class RNLPSFRLibModule extends ReactContextBaseJavaModule implements ActivityEventListener {

    @Nullable
    private static Logcat sLogcat = null;

    private final ReactApplicationContext reactContext;

    public RNLPSFRLibModule(ReactApplicationContext reactContext) {
        super(reactContext);
        reactContext.addActivityEventListener(this);
        this.reactContext = reactContext;
    }

    @NonNull
    @Override
    public String getName() {
        return "RNLPSFRLibModule";
    }

    @ReactMethod
    public void startLogging(@NonNull Promise promise) {
        if (null != sLogcat) {
            try {
                throw new IllegalStateException("Logging already pending...");
            } catch (Throwable throwable) {
                promise.reject(throwable);
                return;
            }
        }

        Log.d(getName(), "startLogging: starting...");
        sLogcat = new Logcat(300000);
        sLogcat.start();
        promise.resolve(true);
    }

    @ReactMethod
    public void isLogging(@NonNull Promise promise) {
        promise.resolve(null != sLogcat);
    }

    @ReactMethod
    public void sendLogs(@NonNull Promise promise) {
        if (null == sLogcat) {
            try {
                throw new IllegalStateException("Logging stopped...");
            } catch (Throwable throwable) {
                promise.reject(throwable);
                return;
            }
        }

        sLogcat.stop();
        sLogcat.toFile(getReactApplicationContext(), (result) -> {
            FileHelper.shareInReports(getCurrentActivity(),
                    reactContext.getPackageName() + ".reports", result,
                    "application/zip",
                    throwable -> {
                        if (null != throwable) promise.reject(throwable);
                        else promise.resolve(true);
                    });

            sLogcat = null;
        });
    }

    @ReactMethod
    public void checkARCoreAvailability(@NonNull Promise promise) {
        promise.resolve(toMap());
    }

    @ReactMethod
    public void isRunningOnEmulator(@NonNull Promise promise) {
        boolean isEmulator = (Build.BRAND.startsWith("generic") && Build.DEVICE.startsWith("generic"))
                || Build.FINGERPRINT.startsWith("generic")
                || Build.FINGERPRINT.startsWith("unknown")
                || Build.HARDWARE.contains("goldfish")
                || Build.HARDWARE.contains("ranchu")
                || Build.MODEL.contains("google_sdk")
                || Build.MODEL.contains("Emulator")
                || Build.MODEL.contains("Android SDK built for x86")
                || Build.MANUFACTURER.contains("Genymotion")
                || Build.PRODUCT.contains("sdk_google")
                || Build.PRODUCT.contains("google_sdk")
                || Build.PRODUCT.contains("sdk")
                || Build.PRODUCT.contains("sdk_x86")
                || Build.PRODUCT.contains("vbox86p")
                || Build.PRODUCT.contains("emulator")
                || Build.PRODUCT.contains("simulator");

        promise.resolve(isEmulator);
    }

    @NonNull
    public WritableMap toMap() {
        WritableMap map = new WritableNativeMap();
        Object ArCoreApkAvailability = checkAvailability();
        if (null != ArCoreApkAvailability) {
            map.putString("name", getString(ArCoreApkAvailability, "name"));
            map.putBoolean("isSupported", getBoolean(ArCoreApkAvailability, "isSupported"));
            map.putBoolean("isTransient", getBoolean(ArCoreApkAvailability, "isTransient"));
            map.putBoolean("isUnknown", getBoolean(ArCoreApkAvailability, "isUnknown"));
            map.putBoolean("isUnsupported", getBoolean(ArCoreApkAvailability, "isUnsupported"));
        }
        return map;
    }

    @Nullable
    String getString(@NonNull Object instance, @NonNull String methodName) {
        Method method = getMethod(instance.getClass(), methodName);
        if (null != method) {
            try {
                Object result = method.invoke(instance);
                if (result instanceof String) {
                    return (String) result;
                }
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            }
        }

        return null;
    }

    boolean getBoolean(@NonNull Object instance, @NonNull String methodName) {
        Method method = getMethod(instance.getClass(), methodName);
        if (null != method) {
            try {
                Object result = method.invoke(instance);
                if (result instanceof Boolean) {
                    return Boolean.TRUE.equals(result);
                }
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                e.printStackTrace();
            }
        }

        return false;
    }

    @Nullable
    private Object ArCoreApk() {
        try {
            Class klass = Class.forName("com.google.ar.core.ArCoreApk");
            Method getInstance = getMethod(klass, "getInstance");

            if (null != getInstance) return getInstance.invoke(null);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Nullable
    private Method getMethod(Class klass, String name) {
        try {
            return klass.getMethod(name);
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Nullable
    private Method getMethodForContext(@NonNull Class klass, @NonNull String name) {
        try {
            return klass.getMethod(name, Context.class);
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Nullable
    private Object checkAvailability() {
        Object instance = ArCoreApk();

        if (null != instance) {
            Method availability = getMethodForContext(instance.getClass(), "checkAvailability");
            if (null != availability) {
                try {
                    return availability.invoke(instance, reactContext.getApplicationContext());
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                }
            }
        }

        return null;
    }

    @Override
    public void onActivityResult(Activity activity, int requestCode, int resultCode, Intent data) {
        FileHelper.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    public void onNewIntent(Intent intent) {

    }
}